unit MainFormUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IdBaseComponent, IdComponent, IdTCPConnection,
  IdTCPClient, IdHTTP, ScktComp;

type
  TFormMain = class(TForm)
    LabelSAU: TLabel;
    EditSAU: TEdit;
    ButtonStatus: TButton;
    ButtonImport: TButton;
    EditQuery: TEdit;
    ButtonSearch: TButton;
    MemoResults: TMemo;
    ClientSocket: TClientSocket;
    LabelUser: TLabel;
    EditUser: TEdit;
    LabelPass: TLabel;
    EditPass: TEdit;
    LabelStorage: TLabel;
    EditStorage: TEdit;
    OpenDialog: TOpenDialog;
    function GetHostFromUrl(URL: String): String;
    function GetPortFromUrl(URL: String): Integer;
    function GetPathFromUrl(URL: String): String;
    procedure ButtonStatusClick(Sender: TObject);
    procedure ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ClientSocketConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ButtonSearchClick(Sender: TObject);
    procedure ButtonImportClick(Sender: TObject);
    function HTMLSpecialChars(Text: String): String;
    procedure SietsAction(SietsCommand, SietsContent: String);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;
  Connecting: Boolean;
  RequestID: Integer;

implementation

{$R *.dfm}

function TFormMain.GetHostFromUrl(URL: String): String;
var B,E: Integer;
begin
Result:='';
B:=Pos('//',URL); E:=0;
if (B>0) then
  begin
  E:=B+2;
  while (URL[E]<>'/') and (E<Length(URL)) do E:=E+1;
  end;
if (B>0) and (E<Length(URL)) then Result:=Copy(URL,B+2,E-B-2);
if (Length(Result)>0) and (Pos(':',Result)>0) then
  Result:=Copy(Result,1,Pos(':',Result)-1);
end;

function TFormMain.GetPortFromUrl(URL: String): Integer;
var B,E: Integer;
    StrRes: String;
begin
Result:=80;
StrRes:='';
B:=Pos('//',URL); E:=0;
if (B>0) then
  begin
  E:=B+2;
  while (URL[E]<>'/') and (E<Length(URL)) do E:=E+1;
  end;
if (B>0) and (E<Length(URL)) then StrRes:=Copy(URL,B+2,E-B-2);
if (Length(StrRes)>0) and (Pos(':',StrRes)>0) then
  Result:=StrToIntDef(Copy(StrRes,Pos(':',StrRes)+1,Length(StrRes)-Pos(':',StrRes)),80);
end;

function TFormMain.GetPathFromUrl(URL: String): String;
var P: Integer;
begin
Result:='';
P:=Pos('//',URL);
while (P>0) do
  begin
  URL[P]:='_'; URL[P+1]:='_';
  P:=Pos('//',URL);
  end;
P:=Pos('/',URL);
if (P>0) then
  Result:=Copy(URL,P,Length(URL)-P+1);
end;

procedure TFormMain.ButtonStatusClick(Sender: TObject);
begin
// Send SIETS status command
SietsAction('status','');
end;

procedure TFormMain.ClientSocketError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
MemoResults.Lines.Append('FATAL ERROR: Connection to '+GetHostFromUrl(EditSAU.Text)+' failed!');
ErrorCode:=0;
Connecting:=False;
end;

procedure TFormMain.ClientSocketConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
Connecting:=False;
end;

procedure TFormMain.ButtonSearchClick(Sender: TObject);
begin
// Send SIETS search command, encode query as valid xml, return 10 documents
SietsAction('search','<query>'+HTMLSpecialChars(EditQuery.Text)+'</query>'#13#10'<docs>10</docs>'#13#10);
end;

procedure TFormMain.ButtonImportClick(Sender: TObject);
var FC: TStringList;
begin
// Send SIETS import command
if OpenDialog.Execute then
  begin
  FC:=TStringList.Create;
  FC.LoadFromFile(OpenDialog.FileName);
  // id is primary key, text - document contents
  SietsAction('insert','<document>'#13#10'<id>'+HTMLSpecialChars(OpenDialog.FileName)+'</id>'#13#10'<text>'+HTMLSpecialChars(FC.Text)+'</text>'#13#10'</document>'#13#10);
  FC.Free;
  end;
end;

function TFormMain.HTMLSpecialChars(Text: String): String;
var I: Integer;
begin
Result:=Text;
I:=1;
while I<Length(Result) do
  begin
  if Result[I]='<' then
    begin
    Result[I]:='&';
    Insert('lt;',Result,I+1);
    I:=I+2;
    end;
  if Result[I]='>' then
    begin
    Result[I]:='&';
    Insert('gt;',Result,I+1);
    I:=I+2;
    end;
  if Result[I]='&' then
    begin
    Result[I]:='&';
    Insert('amp;',Result,I+1);
    I:=I+3;
    end;
  I:=I+1;
  end;
end;

procedure TFormMain.SietsAction(SietsCommand, SietsContent: String);
var Con: Boolean;
    XML,Req,Res: String;
    TF: TextFile;
    I: Integer;
begin
// Prepare for connection
RequestID:=RequestID+1;
Screen.Cursor:=crHourGlass;
MemoResults.Clear;
Connecting:=True;
ClientSocket.Active:=False;
ClientSocket.Host:=GetHostFromUrl(EditSAU.Text);
ClientSocket.Port:=GetPortFromUrl(EditSAU.Text);
// Try to connect
try
  ClientSocket.Active:=True;
  Con:=True;
except
  ClientSocket.Active:=False;
  Con:=False;
end;
// Do not 'hang' during connection progress
while (Connecting) do
  begin
  Sleep(1000);
  Application.ProcessMessages;
  end;
// If connection ok, process with SIETS communications
if (Con) then
  begin
  // Propare XML for POST to api.cgi
  XML:='';
  XML:=XML+'<?xml version="1.0" encoding="US-ASCII"?>'#13#10;
  XML:=XML+'<siets:request xmlns:siets="www.siets.net">'#13#10;
  XML:=XML+'<siets:storage>'+EditStorage.Text+'</siets:storage>'#13#10;
  XML:=XML+'<siets:timestamp>'+DateTimeToStr(Now)+'</siets:timestamp>'#13#10;
  XML:=XML+'<siets:command>'+SietsCommand+'</siets:command>'#13#10;
  XML:=XML+'<siets:requestid>'+IntToStr(RequestID)+'</siets:requestid>'#13#10;
  XML:=XML+'<siets:user>'+EditUser.Text+'</siets:user>'#13#10;
  XML:=XML+'<siets:password>'+EditPass.Text+'</siets:password>'#13#10;
  XML:=XML+'<siets:reply_charset>US-ASCII</siets:reply_charset>'#13#10;
  if (Length(SietsContent)>0) then
    begin
    XML:=XML+'<siets:content>'#13#10;
    XML:=XML+SietsContent;
    XML:=XML+'</siets:content>'#13#10;
    end;
  XML:=XML+'</siets:request>'#13#10;
  // Prepare standard HTTP POST request
  Req:='';
  Req:=Req+'POST '+GetPathFromUrl(EditSAU.Text)+' HTTP/1.0'#13#10;
  Req:=Req+'Host: '+GetHostFromUrl(EditSAU.Text)+#13#10;
  Req:=Req+'User-Agent: SIETS Client Sample'#13#10;
  Req:=Req+'Content-Length: '+IntToStr(Length(XML))+#13#10;
  Req:=Req+#13#10;
  Req:=Req+XML;
  // Dump all request to file
  AssignFile(TF,'request.txt');
  Rewrite(TF);
  Write(TF,Req);
  CloseFile(TF);
  // Send request, wait for response
  ClientSocket.Socket.SendText(Req);
  Sleep(1000);
  Application.ProcessMessages;
  // Read response
  Res:='';
  while (ClientSocket.Socket.ReceiveLength>0) do
    begin
    Res:=Res+ClientSocket.Socket.ReceiveText;
    Sleep(100);
    Application.ProcessMessages;
    end;
  // Dump response to file
  AssignFile(TF,'reply.txt');
  Rewrite(TF);
  Write(TF,Res);
  CloseFile(TF);
  // Replace LF with CRLF
  for I:=1 to Length(Res) do
    if (Res[I]=#10) and (Res[I-1]<>#13) then
      Insert(#13,Res,I);
  // Strip HTTP protocol headers
  if (Pos(#13#10#13#10,Res)>0) then
    Res:=Copy(Res,Pos(#13#10#13#10,Res)+4,Length(Res)-Pos(#13#10#13#10,Res)-4);
  // Display results
  MemoResults.Lines.Append(Res);
  ClientSocket.Active:=False;
  end
else
  // Display error message
  MemoResults.Lines.Append('FATAL ERROR: Connection to '+GetHostFromUrl(EditSAU.Text)+' failed!');
// Wait for connection operation to finish
while (Connecting) do
  begin
  Sleep(1000);
  Application.ProcessMessages;
  end;
// Restore default cursor
Screen.Cursor:=crDefault;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
RequestID:=0;
end;

end.

